# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Postfix::Rate::Policyd

  protected
  #########

  module Daemon

    DEFAULT_UMASK = 0022

    def daemonize(&blk)

     logger.debug "= about to daemonize"
     fork do
        $0 = "postfix-rate-policyd"
        Process.setsid
        exit if fork

        PidFile.store(self, Process.pid)

        Dir.chdir config.data['chdir'] || config.data['application_root']
        File.umask config.data.fetch(:umask, DEFAULT_UMASK)

        redirect_io

        logger.debug "= done fork"
        sleep 0.1
        blk.call
      end
    end

    private
    #######

    def redirect_io(logfile_name = nil)
      begin; STDIN.reopen "/dev/null"; rescue ::Exception; end

      if logfile_name
        begin
          STDOUT.reopen logfile_name, "a"
          File.chmod(0644, logfile_name)
          STDOUT.sync = true
        rescue ::Exception
           begin; STDOUT.reopen "/dev/null"; rescue ::Exception; end
        end
      else
        begin; STDOUT.reopen "/dev/null"; rescue ::Exception; end
      end

      begin; STDERR.reopen STDOUT; rescue ::Exception; end
      STDERR.sync = true
    end

    public
    ######

    #
    # usually called to quit agent
    #
    def daemon_shutdown
      if @daemon
        PidFile.remove(self)
      end
    end

  end

  module PidFile

    def self.pid_filename(daemon)
      daemon.config.data['pid_file']
    end

    def self.store(daemon, pid)
      File.open(pid_filename(daemon), 'w') {|f| f << pid}
    end

    def self.recall(daemon)
      IO.read(pid_filename(daemon)).to_i rescue nil
    end

    def self.remove(daemon)
      FileUtils.rm(pid_filename(daemon), :force => true)
    end


  end
end
