require 'celluloid/io'
require 'celluloid/autostart'
require "celluloid/pool"

module Postfix::Rate::Policyd
  class PostfixServer
    include Celluloid::IO
    include Celluloid::Internals::Logger
    finalizer :shutdown

    DEFAULT_POOL_SIZE = 50

    finalizer :finalize

    def self.start!(host, port, options = {})
      server = new(host, port, options)
      server.start!
      server
    end 

    def initialize(host, port, options = {})
      @host = host
      @port = port
      @pool_size = options.fetch(:pool_size, DEFAULT_POOL_SIZE)
    end

    def start!
      info "- new server on '#{@host}' listening on port: #{@port}."
      @server = TCPServer.new(@host, @port)

      @pool = RequestWorker.pool size: @pool_size

      async.run
    end

    def shutdown
      info "- finalizing server"
      @server.close if @server
    end

    def run
      loop { handle_connection @server.accept }
    end

    def handle_connection(socket)
      @pool.async.process_request(socket)
    end
  end
end